@@ -6,7 +6,7 @@ use strict;
 use 5.010000;
 
 use Exporter qw( import );
-our @EXPORT_OK = qw( map_fmt_to_quality read_config_file set_options );
+our @EXPORT_OK = qw( read_config_file set_options );
 
 use File::Spec::Functions qw( catfile );
 use File::Temp            qw();
@@ -14,13 +14,12 @@ use FindBin               qw( $RealBin $RealScript );
 use List::Util            qw( max );
 use Pod::Usage            qw( pod2usage );
 
-use JSON                   qw();
 use Term::Choose           qw( choose );
 use Term::ReadLine::Simple qw();
 use Text::LineFold         qw();
 
-use App::YTDL::Helper     qw( term_size print_hash encode_fs choose_a_dir choose_a_number insert_sep );
-use App::YTDL::YT_Quality qw( map_fmt_to_quality );
+use App::YTDL::Helper qw( term_size print_hash encode_fs choose_a_dir
+                          choose_a_number insert_sep write_json read_json );
 
 
 
@@ -44,11 +43,11 @@ sub _menus {
         ],
         group_quality => [
             [ 'auto_quality', "- Auto quality mode"   ],
-            [ 'preferred',    "- Preferred qualities" ],
         ],
         group_history => [
-            [ 'log_info',     "- Logging"         ],
-            [ 'max_channels', "- Channel history" ],
+            [ 'log_info',             "- Logging"         ],
+            [ 'max_channels',         "- Channel history" ],
+            [ 'channel_by_timestamp', "- Channel sort"   ],
         ],
         group_directory => [
             [ 'video_dir',     "- Video directory"     ],
@@ -74,30 +73,42 @@ sub set_options {
         push @keys, map { $_->[0] } @{$menus->{$group}};
     }
     my $group = 'main';
+    my $backup_old_idx = 0;
+    my $old_idx = 0;
 
     GROUP: while ( 1 ) {
         my $menu = $menus->{$group};
 
         OPTION: while ( 1 ) {
-            my $back     = '  QUIT';
-            my $continue = $group eq 'main' ? '  CONTINUE' : '  MENU';
-            my @pre  = ( undef, $continue );
+            my $back     = $group eq 'main' ? '  QUIT' : '  <<';
+            my $continue = '  CONTINUE';
+            my @pre  = ( undef, $group eq 'main' ? $continue : () );
             my @real = map( $_->[1], @$menu );
             # Choose
             my $idx = choose(
                 [ @pre, @real ],
-                { prompt => "Options:", layout => 3, index => 1, clear_screen => 1, undef => $back }
+                { prompt => "Options:", layout => 3, index => 1, clear_screen => 1, undef => $back, default => $old_idx }
             );
-            if ( ! defined $idx ) {
-                _write_config_file( $opt, $opt->{config_file}, @keys );
-                exit;
+            if ( ! $idx ) {
+                if ( $group eq 'main' ) {
+                    _write_config_file( $opt, $opt->{config_file}, @keys );
+                    exit;
+                }
+                else {
+                    $old_idx = $backup_old_idx;
+                    $group = 'main';
+                    next GROUP;
+                }
             }
+            if ( $old_idx == $idx ) {
+                $old_idx = 0;
+                next OPTION;
+            }
+            $old_idx = $idx;
             my $choice = $idx <= $#pre ? $pre[$idx] : $menu->[$idx - @pre][0];
-            if ( ! defined $choice ) {
-                _write_config_file( $opt, $opt->{config_file}, @keys );
-                exit;
-            }
             if ( $choice =~ /^group_/ ) {
+                $backup_old_idx = $old_idx;
+                $old_idx = 0;
                 $group = $choice;
                 redo GROUP;
             }
@@ -107,7 +118,6 @@ sub set_options {
                     redo GROUP;
                 }
                 _write_config_file( $opt, $opt->{config_file}, @keys );
-                delete $opt->{change};
                 last GROUP;
             }
             if ( $choice eq "show_help_text" ) {
@@ -147,21 +157,12 @@ sub set_options {
                 my $list = [
                     'choose always manually',
                     'keep choice for the respective Playlist/Channel if possible',
-                    'keep choice always if possible',
+                    'keep choice for the extractor if possible',
                     'use preferred qualities',
                     'use always default (best) quality',
                 ];
                 _opt_choose_from_list( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "preferred" ) {
-                my ( $hash, $keys );
-                my $ref = map_fmt_to_quality();
-                for my $ar ( @$ref ) {
-                    $hash->{$ar->[0]} = $ar->[1];
-                    push @$keys, $ar->[0];
-                }
-                _opt_choose_a_list( $opt, $choice, $hash, $keys );
-            }
             elsif ( $choice eq "retries" ) {
                 my $prompt = 'Download retries';
                 my $digits = 3;
@@ -196,8 +197,16 @@ sub set_options {
                 my $digits = 3;
                 _opt_number_range( $opt, $choice, $prompt, 3 )
             }
+            elsif ( $choice eq "channel_by_timestamp" ) {
+                my $prompt = 'Channel history:';
+                my $list = [
+                    'Sort by name',
+                    'Sort by timestamp'
+                ];
+                _opt_choose_from_list( $opt, $choice, $prompt, $list );
+            }
             elsif ( $choice eq "extractor_dir" ) {
-                my $prompt = 'Use extractor directory';
+                my $prompt = 'Use extractor directories';
                 my $list = [
                     'No',
                     'Yes',
@@ -205,7 +214,7 @@ sub set_options {
                 _opt_choose_from_list( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "channel_dir" ) {
-                my $prompt = 'Use channel directory';
+                my $prompt = 'Use channel directories';
                 my $list = [
                     'No',
                     'If chosen from a channel or list',
@@ -275,7 +284,7 @@ sub _opt_yes_no {
 sub _opt_number_range {
     my ( $opt, $section, $prompt, $digits ) = @_;
     my $current = $opt->{$section};
-    $current = insert_sep( $current ); # $opt->{thsd_sep}
+    $current = insert_sep( $current );
     # Choose_a_number
     my $choice = choose_a_number( $digits, { name => $prompt, current => $current } );
     return if ! defined $choice;
@@ -317,53 +326,6 @@ sub _opt_choose_from_list {
 }
 
 
-sub _opt_choose_a_list {
-    my ( $opt, $section, $ref, $keys ) = @_;
-    my $available = [];
-    my $len_key = max map length, @$keys;
-    for my $key ( @$keys ) {
-        push @$available, sprintf "%*d => %s", $len_key, $key, $ref->{$key};
-    }
-    my $current = $opt->{$section} // [];
-    my $new     = [];
-    my $key_cur = 'Current > ';
-    my $key_new = '    New > ';
-    my $l_k     = length $key_cur > length $key_new ? length $key_cur : length $key_new;
-    my $lf      = Text::LineFold->new( %{$opt->{line_fold}} );
-    $lf->config( 'ColMax', ( term_size() )[0] );
-    while ( 1 ) {
-        my $confirm = 'CONFIRM';
-        my $prompt = $key_cur . join( ', ', @$current ) . "\n";
-        $prompt   .= $key_new . join( ', ', @$new )     . "\n\n";
-        $prompt   .= 'Choose:';
-        # Choose
-        my $val = choose(
-            [ undef, $confirm, map( "  $_", @$available ) ],
-            { prompt => $prompt, lf => [0,$l_k], layout => 3, clear_screen => 1, undef => 'BACK' }
-        );
-        if ( ! defined $val ) {
-            if ( @$new ) {
-                $new = [];
-                next;
-            }
-            else {
-                return;
-            }
-        }
-        if ( $val eq $confirm ) {
-            if ( @$new ) {
-                $opt->{$section} = $new;
-                $opt->{change}++;
-            }
-            return;
-        }
-        $val =~ s/^\s+//;
-        $val = ( split / => /, $val )[0];
-        push @$new, $val;
-    }
-}
-
-
 sub _write_config_file {
     my ( $opt, $file, @keys ) = @_;
     return if ! $opt->{change};
@@ -371,37 +333,19 @@ sub _write_config_file {
     for my $section ( sort @keys ) {
         $tmp->{$section} = $opt->{$section};
     }
-    _write_json( $file, $tmp );
+    write_json( $file, $tmp );
+    delete $opt->{change};
 }
 
 
 sub read_config_file {
     my ( $opt, $file ) = @_;
-    my $tmp = _read_json( $file );
-    for my $section ( keys %$tmp ) {
+    my $tmp = read_json( $file );
+    my @keys = keys %$tmp;
+    for my $section ( @keys ) {
         $opt->{$section} = $tmp->{$section};
     }
-    return;
-}
-
-
-sub _write_json {
-    my ( $file, $h_ref ) = @_;
-    my $json = JSON::XS->new->pretty->utf8->encode( $h_ref );
-    open my $fh, '>', encode_fs( $file ) or die $!;
-    print $fh $json;
-    close $fh;
-}
-
-
-sub _read_json {
-    my ( $file ) = @_;
-    return {} if ! -f encode_fs( $file );
-    open my $fh, '<', encode_fs( $file ) or die $!;
-    my $json = do { local $/; <$fh> };
-    close $fh;
-    my $h_ref = JSON::XS->new->pretty->utf8->decode( $json ) if $json;
-    return $h_ref;
+    return @keys;
 }
 
 
