@@ -115,9 +115,6 @@ sub json_to_hash {
     }
     $tmp->{$video_id}{fmt_to_info} = $formats;
     _prepare_info_hash( $tmp, $video_id );
-    if ( defined $tmp->{$video_id}{extractor_key} && $tmp->{$video_id}{extractor_key} =~ /^youtube\z/i ) {
-        $tmp->{$video_id}{youtube} = 1;
-    }
     return;
 }
 
@@ -150,7 +147,7 @@ sub _prepare_info_hash {
     }
     $info->{$video_id}{uploader}      //= $info->{$video_id}{uploader_id};
     $info->{$video_id}{extractor}     //= $info->{$video_id}{extractor_key};
-    $info->{$video_id}{extractor_key} //= $info->{$video_id}{extractor};
+    $info->{$video_id}{extractor_key} //= $info->{$video_id}{extractor} // '';
 }
 
 
