@@ -170,11 +170,12 @@ sub _download_video {
 sub _log_info {
     my ( $opt, $info, $video_id ) = @_;
     my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
+    my $id = $info->{$video_id}{extractor_key} eq 'Youtube' ? $video_id : $info->{$video_id}{extractor_key} . ' ' . $video_id;
     my $log_str = sprintf( "%04d-%02d-%02d %02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min )
-        . '>  ' . sprintf( "%11s", $info->{$video_id}{youtube} ? $video_id : ( $info->{$video_id}{extractor_key} // '' ) . ' ' . ( $info->{$video_id}{id} // '' ) )
+        . '>  ' . sprintf( "%11s", $id )
         . ' | ' . ( $info->{$video_id}{uploader_id} // 'xxxxx' )
         . ' | ' . ( $info->{$video_id}{playlist_id} // '-----' )
-        . ' | ' . ( $info->{$video_id}{upload_date} // '0000-00-00' )
+        . ' | ' . ( $info->{$video_id}{upload_date} )
         . '   ' . basename( $info->{$video_id}{file_name} );
     open my $log, '>>:encoding(UTF-8)', encode_fs( $opt->{log_file} ) or die $!;
     flock $log, LOCK_EX     or die $!;
