@@ -17,9 +17,8 @@ use Unicode::GCString qw();
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
-use App::YTDL::YT_Quality qw( map_fmt_to_quality );
 use App::YTDL::Info_Print qw( linefolded_print_info );
-use App::YTDL::Helper     qw( term_size unicode_trim encode_stdout_lax encode_fs );
+use App::YTDL::Helper     qw( term_size unicode_trim encode_stdout_lax encode_fs read_json write_json );
 
 
 
@@ -42,8 +41,7 @@ sub get_download_infos {
     my $count = 0;
     $opt->{up} = 0;
 
-    VIDEO:
-    while ( @video_ids ) {
+    VIDEO: while ( @video_ids ) {
         my $video_id = shift @video_ids;
         $count++;
         my $key_len = 13;
@@ -68,16 +66,17 @@ sub get_download_infos {
             next VIDEO;
         }
         $fmt = _fmt_quality( $opt, $info, $fmt, $video_id );
-        print up( $opt->{up} ), cldown;
-        $opt->{up} = 0;
         if ( ! defined $fmt ) {
-            my ( $delete, $append, $redo ) = ( 'Delete', 'Append', 'Redo' );
+            my ( $delete, $append, $redo, $pref_qualities, $quit ) = ( 'Delete', 'Append', 'Redo', 'Preferred qualities', 'Quit' );
             # Choose
             my $choice = choose(
-                [ undef, $delete, $append, $redo ],
-                { prompt => 'Your choice: ', undef => 'QUIT' }
+                [ undef, $delete, $append, $redo, $pref_qualities, $quit ],
+                { prompt => 'Your choice: ', undef => 'Back' }
             );
             if ( ! defined $choice ) {
+                unshift @video_ids, $video_id;
+            }
+            elsif ( $choice eq $quit ) {
                 print locate( 1, 1 ), cldown;
                 say "Quit";
                 exit;
@@ -95,62 +94,75 @@ sub get_download_infos {
             elsif ( $choice eq $redo ) {
                 unshift @video_ids, $video_id;
             }
+            elsif ( $choice eq $pref_qualities ) {
+                _set_preferred_qualities( $opt, $info, $video_id );
+                unshift @video_ids, $video_id;
+            }
+            print up( $opt->{up} ), cldown;
+            $opt->{up} = 0;
             $count--;
             next VIDEO;
         }
-        else {
-            my $video_dir = $opt->{video_dir};
-            if ( $opt->{extractor_dir} ) {
-                if ( $info->{$video_id}{extractor_key} ) {
-                    my $extractor_dir = $info->{$video_id}{extractor_key};
-                    $extractor_dir =~ s/\s/_/g;
-                    $video_dir = catdir $video_dir, $extractor_dir;
-                    mkdir encode_fs( $video_dir ) or die $! if ! -d encode_fs( $video_dir );
-                }
+        print up( $opt->{up} ), cldown;
+        $opt->{up} = 0;
+        my $video_dir = $opt->{video_dir};
+        if ( $opt->{extractor_dir} ) {
+            if ( $info->{$video_id}{extractor_key} ) {
+                my $extractor_dir = $info->{$video_id}{extractor_key};
+                $extractor_dir =~ s/\s/_/g;
+                $video_dir = catdir $video_dir, $extractor_dir;
+                mkdir encode_fs( $video_dir ) or die $! if ! -d encode_fs( $video_dir );
             }
-            if ( $opt->{channel_dir} == 2 || $opt->{channel_dir} == 1 && $info->{$video_id}{from_list} ) {
-                if ( $info->{$video_id}{uploader} ) {
-                    my $channel_name = $info->{$video_id}{uploader};
-                    $channel_name =~ s/\s/_/g;
-                    $video_dir = catdir $video_dir, $channel_name;
-                    mkdir encode_fs( $video_dir ) or die $! if ! -d encode_fs( $video_dir );
-                }
+        }
+        if ( $opt->{channel_dir} == 2 || $opt->{channel_dir} == 1 && $info->{$video_id}{from_list} ) {
+            if ( $info->{$video_id}{uploader} ) {
+                my $channel_name = $info->{$video_id}{uploader};
+                $channel_name =~ s/\s/_/g;
+                $video_dir = catdir $video_dir, $channel_name;
+                mkdir encode_fs( $video_dir ) or die $! if ! -d encode_fs( $video_dir );
             }
-            $info->{$video_id}{video_url} = $info->{$video_id}{fmt_to_info}{$fmt}{url};
-            $info->{$video_id}{file_name} = catfile( $video_dir, _get_filename( $opt, $info, $fmt, $video_id ) );
-            $info->{$video_id}{count}     = $count;
-            $info->{$video_id}{fmt}       = $fmt;
-            $print_array->[0] =~ s/\n\z/ ($fmt)\n/;
-            unshift @$print_array, sprintf "%*.*s : %s\n", $key_len, $key_len, 'video', $count;
-            binmode STDOUT, ':pop';
-            print for map { encode_stdout_lax( $_ ) } @$print_array;
-            binmode STDOUT, ':encoding(console_out)';
-            print "\n";
-            if ( $opt->{max_channels} && $info->{$video_id}{uploader_id} ) {
-                if ( $info->{$video_id}{extractor_key} eq 'Youtube' ) {
-                    my $channel    = $info->{$video_id}{uploader};
-                    my $channel_id = $info->{$video_id}{uploader_id};
-                    if ( none{ $channel_id eq ( split /,/, $_ )[1] } @{$opt->{channel_sticky}} ) {
-                        my $idx = first_index { $channel_id eq ( split /,/, $_ )[1] } @{$opt->{channel_history}};
-                        if ( $idx > -1 ) {
-                            splice @{$opt->{channel_history}}, $idx, 1;
-                        }
-                        unshift @{$opt->{channel_history}}, sprintf "%s,%s", $channel, $channel_id;
-                    }
+        }
+        $info->{$video_id}{video_url} = $info->{$video_id}{fmt_to_info}{$fmt}{url};
+        $info->{$video_id}{file_name} = catfile( $video_dir, _get_filename( $opt, $info, $fmt, $video_id ) );
+        $info->{$video_id}{count}     = $count;
+        $info->{$video_id}{fmt}       = $fmt;
+        $print_array->[0] =~ s/\n\z/ ($fmt)\n/;
+        unshift @$print_array, sprintf "%*.*s : %s\n", $key_len, $key_len, 'video', $count;
+        binmode STDOUT, ':pop';
+        print for map { encode_stdout_lax( $_ ) } @$print_array;
+        binmode STDOUT, ':encoding(console_out)';
+        print "\n";
+        if ( $opt->{max_channels} && $info->{$video_id}{uploader_id} ) {
+            my $channel       = $info->{$video_id}{uploader};
+            my $channel_id    = $info->{$video_id}{uploader_id};
+            my $extractor_key = $info->{$video_id}{extractor_key};
+            my $channel_url;
+            $channel_url = 'c#'                if $info->{$video_id}{extractor_key} eq 'Youtube';
+            $channel_url = 'http://vimeo.com/' if $info->{$video_id}{extractor_key} eq 'Vimeo';
+            if ( defined $channel_url ) {
+                $channel_url .= $channel_id;
+                if ( none{ $channel_url eq $_ } keys %{$opt->{channel_sticky}} ) {
+                    $opt->{channel_history}{$channel_url} = {
+                        channel_url   => $channel_url,
+                        channel       => $channel,
+                        channel_id    => $channel_id,
+                        extractor_key => $extractor_key,
+                        timestamp     => time(),
+                    };
                 }
             }
         }
     }
     print "\n";
     if ( $opt->{max_channels} ) {
-        while ( @{$opt->{channel_history}} > $opt->{max_channels} ) {
-            pop @{$opt->{channel_history}};
+        my $c_hist = $opt->{channel_history};
+        my @keys = sort { $c_hist->{$b}{timestamp} <=> $c_hist->{$a}{timestamp} } keys %$c_hist;
+        while ( @keys > $opt->{max_channels} ) {
+            my $key = pop @keys;
+            delete $c_hist->{$key};
+            @keys = sort { $c_hist->{$b}{timestamp} <=> $c_hist->{$a}{timestamp} } keys %$c_hist;
         }
-        open my $fh, '>:encoding(UTF-8)', encode_fs( $opt->{c_history_file} ) or die $!;
-        for my $line ( @{$opt->{channel_history}} ) {
-            say $fh $line;
-        }
-        close $fh;
+        write_json( $opt->{h_channel_file}, $c_hist );
     }
     $opt->{total_nr_videos} = $count;
     if ( ! $opt->{total_nr_videos} ) {
@@ -162,6 +174,64 @@ sub get_download_infos {
 }
 
 
+sub _set_preferred_qualities {
+    my ( $opt, $info, $video_id ) = @_;
+    my $fmt_to_info = $info->{$video_id}{fmt_to_info};
+    my $ex_key = $info->{$video_id}{extractor_key};
+    my ( @qualities, @format_ids );
+    if ( $ex_key eq 'Youtube' ) {
+        for my $fmt ( sort { $a <=> $b } keys %$fmt_to_info ) {
+            if ( $fmt_to_info->{$fmt}{format} =~ /^\Q$fmt\E\s*-\s*(.+)\z/ ) {
+                push @qualities, sprintf '%3s - %s %s', $fmt, $1, $fmt_to_info->{$fmt}{ext};
+            }
+            else {
+                push @qualities, $fmt_to_info->{$fmt}{format} . ' ' . $fmt_to_info->{$fmt}{ext};
+            }
+            push @format_ids, $fmt;
+        }
+    }
+    else {
+        for my $fmt ( sort { $a cmp $b } keys %$fmt_to_info ) {
+            push @qualities, $fmt_to_info->{$fmt}{format} . ' ' . $fmt_to_info->{$fmt}{ext};
+            push @format_ids, $fmt;
+        }
+    }
+    my $pref_qual = read_json( $opt->{preferred_file} );
+
+    CHOICE: while ( 1 ) {
+        my @pre = ( undef );
+        my @choices = map { sprintf '%d. choice: %s', $_, $pref_qual->{$ex_key}[$_ - 1] // '' } 1 .. 3;
+        my $idx_choice = choose(
+            [ @pre, @choices ],
+            { prompt => "Set preferred qualities ($ex_key):", layout => 3, index => 1, undef => 'Back' }
+        );
+        if ( ! $idx_choice ) {
+            write_json( $opt->{preferred_file}, $pref_qual );
+            return;
+        }
+        $idx_choice -= @pre;
+        @pre = ( undef );
+        my $prompt = "Set preferred qualities ($ex_key):\n$choices[$idx_choice]";
+        # Choose
+        my $fmt_res_idx = choose(
+            [ @pre, @qualities ],
+            { prompt => $prompt, index => 1, order => 1, undef => 'Back' }
+        );
+        if ( ! $fmt_res_idx ) {
+            next CHOICE;
+        }
+        else {
+            $fmt_res_idx -= @pre;
+            my $fmt = $format_ids[$fmt_res_idx];
+            $pref_qual->{$ex_key}[$idx_choice] = $fmt;
+            next CHOICE;
+        }
+    }
+}
+
+
+
+
 sub _get_filename {
     my ( $opt, $info, $fmt, $video_id ) = @_;
     my $title  = $info->{$video_id}{title};
@@ -185,7 +255,6 @@ sub _get_filename {
 sub _fmt_quality {
     my ( $opt, $info, $fmt, $video_id ) = @_;
     my $auto_quality = $opt->{auto_quality};
-    $auto_quality = 2 if $auto_quality == 3 && $info->{$video_id}{extractor_key} ne 'Youtube';
     my $fmt_ok;
     if ( $auto_quality == 0 ) {
     }
@@ -203,19 +272,22 @@ sub _fmt_quality {
             $fmt_ok = 1;
         }
     }
-    elsif ( $auto_quality == 2 ) {
-        if ( ! defined $opt->{ap_key} ) {
+    elsif ( $auto_quality == 2 && $info->{$video_id}{extractor_key} ) {
+        my $pq_key = $info->{$video_id}{extractor_key};
+        if ( ! defined $opt->{$pq_key} ) {
             $fmt = _choose_fmt( $opt, $info, $video_id );
             return if ! defined $fmt;
-            $opt->{ap_key} = $fmt;
+            $opt->{$pq_key} = $fmt;
         }
         else {
             $fmt = $opt->{ap_key};
         }
         $fmt_ok = 1;
+
     }
-    elsif ( $auto_quality == 3 ) {
-        my @pref_qualities = @{$opt->{preferred}//[]};
+    elsif ( $auto_quality == 3 && $info->{$video_id}{extractor_key} ) {
+        my $pref_qual = read_json( $opt->{preferred_file} );
+        my @pref_qualities = @{$pref_qual->{$info->{$video_id}{extractor_key}}};
         for my $pq ( @pref_qualities ) {
             if ( any { $pq eq $_ } keys %{$info->{$video_id}{fmt_to_info}} ) {
                 $fmt = $pq;
