@@ -46,7 +46,7 @@ sub _youtube_print_info {
 
 sub _prepare_print_info {
     my ( $opt, $info, $video_id ) = @_;
-    if ( $info->{$video_id}{youtube} ) {
+    if ( $info->{$video_id}{extractor_key} eq 'Youtube' ) {
         _youtube_print_info( $opt, $info, $video_id );
     }
     $info->{$video_id}{published} = $info->{$video_id}{upload_date};
@@ -63,7 +63,7 @@ sub _prepare_print_info {
         $opt->{up} = 0;
         splice @keys, 1, 0, 'status', 'errorcode', 'reason';
     }
-    push @keys, 'extractor' if ! $info->{$video_id}{youtube};
+    push @keys, 'extractor' if $info->{$video_id}{extractor} !~ /^youtube\z/i;
     push @keys, qw( author duration raters avg_rating view_count published content description keywords );
     for my $key ( @keys ) {
         next if ! $info->{$video_id}{$key};
