@@ -42,7 +42,8 @@ sub from_arguments_to_choices {
             _choose_videos_and_add_to_info_hash( $opt, $info, $tmp, $is_youtube );
         }
         elsif ( my $video_id = _video_id( $opt, $id )  ) {
-            $info->{$video_id}{youtube} = 1;
+            $info->{$video_id}{extractor}     = 'youtube';
+            $info->{$video_id}{extractor_key} = 'Youtube';
         }
         else {
             my $tmp = _non_yt_id_to_tmp_info_hash( $opt, $id );
@@ -51,7 +52,6 @@ sub from_arguments_to_choices {
             if ( @keys == 1 ) {
                 my $video_id = $keys[0];
                 $info->{$video_id} = $tmp->{$video_id};
-                $info->{$video_id}{youtube} = 0;
             }
             else {
                 _choose_videos_and_add_to_info_hash( $opt, $info, $tmp, $is_youtube );
@@ -252,7 +252,10 @@ sub _choose_videos_and_add_to_info_hash {
                 next VIDEO_ID;
             }
             $tmp->{$video_id}{from_list} = 1;
-            $tmp->{$video_id}{youtube}   = $is_youtube;
+            if ( $is_youtube ) {
+                $tmp->{$video_id}{extractor}     = 'youtube';
+                $tmp->{$video_id}{extractor_key} = 'Youtube';
+            }
             push @videos, sprintf "%11s | %7s  %10s  %s", $video_id, $tmp->{$video_id}{duration}, $tmp->{$video_id}{upload_date}, $title;
             push @tmp_video_ids, $video_id;
             $index++;
